import bpy
import os
import bpy.utils.previews
import shutil
from bpy.types import Operator
from bpy_extras.io_utils import ImportHelper
from . import bagapie_geopack_icon
from .utils import Get_addon_pref

previews = bpy.utils.previews.new()

def import_icons():
    global previews

    pref = Get_addon_pref()
    filepath = pref.geopack_packs_location
    
    for  dirpath, dirnames, filenames in os.walk(filepath):
        # Vérifie si 'geopack.config' est dans la liste des fichiers du dossier courant
        if 'geopack.config' in filenames:
            # Parcours tous les fichiers dans le dossier courant
            for file in os.listdir(dirpath):
                # Construit le chemin complet du fichier
                file_path = os.path.join(dirpath, file)
                # Vérifie si le chemin est un fichier et si l'extension est celle d'une image
                if os.path.isfile(file_path) and file.lower().endswith(('.png', '.jpg', '.jpeg', '.bmp', '.tif', '.tiff')):
                    icon_name = os.path.splitext(file)[0]
                    if icon_name not in bagapie_geopack_icon.previews:
                        previews.load(icon_name, file_path, 'IMAGE')
    
    # Parcourir chaque fichier dans le sous-dossier 'geopack_icons' du répertoire de l'addon
    
    # for file in os.listdir(os.path.join(os.path.dirname(__file__), 'geopack_icons')):
    #     if file.lower().endswith(('.png', '.jpg', '.jpeg', '.bmp', '.tif', '.tiff')):
    #         icon_name = os.path.splitext(file)[0]
    #         file_path = os.path.join(os.path.dirname(__file__), 'geopack_icons', file)
    #         previews.load(icon_name, file_path, 'IMAGE')


def unload_icons():
    global previews
    try:
        bpy.utils.previews.remove(previews)
    except:
        pass


# POUR LA DEF :
# from . import bagapie_geopack_icon

# POUR AFFICHER LES PREVIEWS :
# col.template_icon(icon_value = bagapie_geopack_icon.previews[nom_image].icon_id, scale=10)



class BAGAPIE_OT_add_icon(Operator, ImportHelper):
    bl_idname = "bagapie.add_icon"
    bl_label = "Copier l'image dans le dossier du GeoPack"

    filter_glob: bpy.props.StringProperty(
        default="*.png;*.jpg;*.jpeg;*.bmp;*.tif;*.tiff",
        options={'HIDDEN'},
    )

    target_folder: bpy.props.StringProperty(
        name="Dossier Cible",
        default="//",
        subtype='DIR_PATH',
        options={'HIDDEN'},
    )
    img_mo_name: bpy.props.StringProperty(
        name="None",
        default="",
        options={'HIDDEN'},
    )

    def execute(self, context):
        filepath = self.properties.filepath #file selected by user

        shutil.copy(filepath, self.target_folder) #copy file

        # Obtien nom du fichier sans le chemin et son extension
        original_name, file_extension = os.path.splitext(os.path.basename(filepath))
        
        new_name = self.img_mo_name + file_extension
        
        current_file_path = os.path.join(self.target_folder, original_name + file_extension)
        
        new_file_path = os.path.join(self.target_folder, new_name)
        
        os.rename(current_file_path, new_file_path)
        
        import_icons()

        return {'FINISHED'}

class BAGAPIE_OT_remove_icon(Operator):
    bl_idname = "bagapie.remove_icon"
    bl_label = "Delete Icon"

    target_folder: bpy.props.StringProperty(
        name="Dossier Cible",
        default="//",
        subtype='DIR_PATH'
    )
    img_mo_name: bpy.props.StringProperty(
        name="image_name",
        default="",
    )

    def execute(self, context):
        image_extensions = ['.png', '.jpg', '.jpeg', '.bmp', '.tif', '.tiff']

        abs_target_folder = bpy.path.abspath(self.target_folder) # absolute path
        file_path = os.path.join(abs_target_folder, self.img_mo_name) # complete path

        for ext in image_extensions:
            file_path = os.path.join(abs_target_folder, self.img_mo_name + ext)
            if os.path.isfile(file_path):
                os.remove(file_path)
                break



        # os.remove(file_path)
        
        unload_icons()
        import_icons()

        return {'FINISHED'}


classes = [
    BAGAPIE_OT_add_icon,
    BAGAPIE_OT_remove_icon,
]